from cmdMenuFramework.Instruction import Instruction
from cmdMenuFramework.InstType import InstType
from cmdMenuFramework.Printer import Printer
class Set(Instruction):

        
    def __init__(self, context, name, content):
        super(Set, self).__init__(context, InstType.set, name, content)
        
    def execute(self):
        
        self.context.path.addInstruction(self)
        
        if type(self.content) != dict:
            Printer.printError ("Set instruction expected a dictionnary, got "+str(type(self.content))+" instead", self.context)
            self.context.path.goBack()
            return None
        
        # look every value to set, and execute
        for varName in self.content:
            varContent = self.content[varName]
            # if content is none
            if varContent is None:
                self.context.vars[varName] = None
                
            # if content is dict: interpret it
            elif type(varContent) == dict:
                if (len(varContent) != 1):
                    s1 = 'when setting user variable "'+varName+'"'
                    Printer.printError ("Multiple embedded instructions "+s1+": use a block instruction instead", self.context)
                    continue
                for id in varContent:
                    i = Instruction.letterToInstruction(id[:1], id, varContent[id], self.context)
                    if i is None: 
                        break
                    output = i.execute()
                    self.context.vars[varName] = output
                    
            # else
            else:
                self.context.vars[varName] = varContent
            
        self.context.path.goBack()
        


from cmdMenuFramework.Instruction import Instruction
from cmdMenuFramework.InstType import InstType

class Loop(Instruction):
    
    def __init__(self, context, name, content):
        super(Loop, self).__init__(context, InstType.loop, name, content)
        
    def execute(self):
        
        self.context.path.addInstruction(self)

        while not self.context.finish==True:
            for id in self.content:
                
                if self.context.exit == True:
                    self.context.exit = False
                    self.context.finish = True
                    break
                
                i = Instruction.letterToInstruction(id[:1], id, self.content[id], self.context)
                
                if i is None: 
                    continue
                i.execute()
        
        self.context.finish = False
        self.context.path.goBack()
        return self.context.output


from cmdMenuFramework.Instruction import Instruction
from cmdMenuFramework.InstType import InstType
from cmdMenuFramework.Printer import Printer

class SetDefault(Instruction):

        
    def __init__(self, context, name, content):
        super(SetDefault, self).__init__(context, InstType.setDefault, name, content)
        
    def execute(self):
        
        self.context.path.addInstruction(self)
        
        if type(self.content) != dict:
            Printer.printError ("SetDefault instruction expected a dictionnary, got "+str(type(self.content))+" instead", self.context)
            self.context.path.goBack()
            return None
        
        # look every value to set, and execute
        for varName in self.content:
            varContent = self.content[varName]
            # if content is none
            if varContent is None:
                self.context.vars[varName] = None
                
            # if content is dict: interpret it
            elif type(varContent) == dict:
                if (len(varContent) != 1):
                    s1 = 'when setting default variable "'+varName+'"'
                    Printer.printError ("Multiple embedded instructions "+s1+": use a block instruction instead", self.context)
                    continue
                for id in varContent:
                    i = Instruction.letterToInstruction(id[:1], id, varContent[id], self.context)
                    if i is None: 
                        break
                    output = i.execute()
                    
            # else
            else:
                output = varContent
            
            allDefNames = [
                'file' ,
                'controllerFile' ,
                'controllerClass',
                'input' ,
                'exit',
                'exception',
                'finish',
                'choicePrompt',
                'inputPrompt',
                'indent',
                'availableChoices',
                'indentWithPipes',
                'showTrace'
            ]
            
            if varName not in allDefNames:
                Printer.printError ('"'+varName+"\" is not a default menu variable! Oh no!  :'(", self.context)
                continue
            
            if varName == 'file':
                if (t := type(output)) == str:
                    self.context.file = output
                else:
                    Printer.printError ("'file' default variable expects a string, got "+str(t)+" instead", self.context)
                    continue
            
            if varName == 'controllerFile':
                if (t := type(output)) == str:
                    self.context.controllerFile = output
                else:
                    Printer.printError ("'controllerFile' default variable expects a string, got "+str(t)+" instead", self.context)
                    continue
            
            if varName == 'controllerClass':
                if (t := type(output)) == str:
                    self.context.controllerClass = output
                else:
                    Printer.printError ("'controllerClass' default variable expects a string, got "+str(t)+" instead", self.context)
                    continue
            
            if varName == 'input':
                if (t := type(output)) == str:
                    self.context.input = output
                else:
                    Printer.printError ("'input' default variable expects a string, got "+str(t)+" instead", self.context)
                    continue
            
            if varName == 'output':
                self.context.output = output
            
            if varName == 'userChoice':
                if (t := type(output)) == str:
                    self.context.userChoice = output
                else:
                    Printer.printError ("'userChoice' default variable expects a string, got "+str(t)+" instead", self.context)
                    continue
            
            if varName == 'exit':
                if (t := type(output)) == bool:
                    self.context.exit = output
                else:
                    Printer.printError ("'exit' default variable expects a boolean, got "+str(t)+" instead", self.context)
                    continue
            
            if varName == 'exception':
                if (t := type(output)) == str:
                    self.context.exception = output
                else:
                    Printer.printError ("'exception' default variable expects a string, got "+str(t)+" instead", self.context)
                    continue
            
            if varName == 'finish':
                if (t := type(output)) == bool:
                    self.context.finish = output
                else:
                    Printer.printError ("'finish' default variable expects a boolean, got "+str(t)+" instead", self.context)
                    continue
            
            if varName == 'choicePrompt':
                if (t := type(output)) == str:
                    self.context.choicePrompt = output
                else:
                    Printer.printError ("'choicePrompt' default variable expects a string, got "+str(t)+" instead", self.context)
                    continue
            
            if varName == 'inputPrompt':
                if (t := type(output)) == str:
                    self.context.inputPrompt = output
                else:
                    Printer.printError ("'inputPrompt' default variable expects a string, got "+str(t)+" instead", self.context)
                    continue
            
            if varName == 'indent':
                if (t := type(output)) == bool:
                    self.context.indent = output
                else:
                    Printer.printError ("'indent' default variable expects a boolean, got "+str(t)+" instead", self.context)
                    continue
            
            if varName == 'indentWithPipes':
                if (t := type(output)) == bool:
                    self.context.indentWithPipes = output
                else:
                    Printer.printError ("'indentWithPipes' default variable expects a boolean, got "+str(t)+" instead", self.context)
                    continue
            
            if varName == 'showTrace':
                if (t := type(output)) == bool:
                    self.context.showTrace = output
                else:
                    Printer.printError ("'showTrace' default variable expects a boolean, got "+str(t)+" instead", self.context)
                    continue
            
            
        self.context.path.goBack()
        


from cmdMenuFramework.Instruction import Instruction
from cmdMenuFramework.InstType import InstType
from cmdMenuFramework.Printer import Printer

class Choice(Instruction):
    
    def __init__(self, context, name, content):
        super(Choice, self).__init__(context, InstType.choice, name, content)
        
    def execute(self):
        """Runs the instruction. 
        
        Stores the choice in the context as available, and displays it.
        Does not return anything."""
        
        self.context.path.addInstruction(self)
        
        #get name of the choice
        self.choiceName = None
        for id in self.content:
            letter1 = id[:1]
            if letter1 == 'n':
                nameInstruction = Instruction.letterToInstruction(id[:1], id, self.content[id], self.context)
                self.choiceName = nameInstruction.execute()
                break
        
        if self.choiceName is None:
            Printer.printError("This choice does not have a name instruction", self.context)
        
        #add to list of choices
        self.context.availableChoices.append(self)
        
        # print choice
        Printer.printChoice(self, self.context)
        
        self.context.path.goBack()
        
        
        
    def executeContent(self):
        self.context.path.addInstruction(self)
        
        if len(self.content)>2:
            Printer.printError ("Group the instructions inside a choice into a block instruction if there are multiple (except for the name instruction)", self.context)

        
        for id in self.content:
            letter1 = id[:1]
            if letter1 != 'n':
                instruction = Instruction.letterToInstruction(id[:1], id, self.content[id], self.context)
                output = instruction.execute()
                self.context.path.goBack()
                return output
            
        self.context.path.goBack()
        return None
        
        


from cmdMenuFramework.Instruction import Instruction
from cmdMenuFramework.InstType import InstType
from cmdMenuFramework.Printer import Printer

class Exception(Instruction):
    
    def __init__(self, context, name, content):
        super(Exception, self).__init__(context, InstType.exception, name, content)
         
    
    def execute(self):
        self.context.path.addInstruction(self)
        
        msg = None
        
        if self.content is not None: 
            if type(self.content) == dict:
                if (len(self.content) != 1):
                    Printer.printError ("Multiple embedded instructions: use a block instruction instead", self.context)
                    self.context.path.goBack()
                    return None
            
                for id in self.content:
                    i = Instruction.letterToInstruction(id[:1], id, self.content[id], self.context)
                    if i is None: 
                        self.context.path.goBack()
                        return None
                    msg = i.execute()
                    if type(self.content) != str:
                        Printer.printError('Exception instruction expects a string (the message to display), got '+type(self.content)+' from embedded instruction instead.')
                        self.context.path.goBack()
                        return None
            
            elif type(self.content) == str:
                msg = self.content
            
            else: 
                self.context.path.goBack()
                Printer.printError('Exception instruction expects a string (the message to display), got '+type(self.content)+' instead.')
                return None
        
        else: 
            # return the last exception
            self.context.path.goBack()
            return self.context.exception    
            
        Printer.printError(msg, self.context)
        self.context.path.goBack()
        return msg


from cmdMenuFramework.Instruction import Instruction
from cmdMenuFramework.InstType import InstType
from cmdMenuFramework.Printer import Printer
class Get(Instruction):
    
    def __init__(self, context, name, content):
        super(Get, self).__init__(context, InstType.get, name, content)
        
    def execute(self):
        
        self.context.path.addInstruction(self)
        
        # find var name
        if self.content is None:
            self.context.path.goBack()
            return None
        
        if (type(self.content)) == dict:
            if (len(self.content) != 1):
                Printer.printError ("Multiple embedded instructions: use a block instruction instead", self.context)
                self.context.path.goBack()
                return None
            
            for id in self.content:
                i = Instruction.letterToInstruction(id[:1], id, self.content[id], self.context)
                if i is None: 
                    self.context.path.goBack()
                    return None
                varName = i.execute()
                if type(varName) != str:  
                    Printer.printError ("Get instruction expected a variable name (str), got "+ type(varName) +" from embedded instruction instead", self.context)
                    self.context.path.goBack() 
                    return None
                
        elif type(self.content) == str:   
            varName = self.content
        
        
        else: 
            Printer.printError ("Get instruction expected a variable name (str), got "+ type(self.content) +" instead", self.context)
            self.context.path.goBack() 
            return None
        
        
        # get var from context
        
        if varName == '*':
            self.context.path.goBack()
            return self.context.vars
        
        try :
            var = self.context.vars[varName]
        except KeyError :
            var = None
            
        self.context.path.goBack()
        return var


from cmdMenuFramework.Menu import Menu


def startMenu(fileName : str):
    """
    Launches the menu that is described by the given yaml file, in the cmd prompt. 
    If the file is not found, throws an exception, and terminates.
    """
    menu = Menu(fileName)
    menu.start()




from cmdMenuFramework.Context import Context

from colorama import Fore, Style, Back



class Printer: 
    """The class responsible for prints."""
    
    def prepareIndent(context, forChoice = False):
        # prepare the indent
        # print ('#### '+str(context.path.getNbChoicesMade()))
        prefix = ""
        if context.indent == True:
            
            r = context.path.getNbChoicesMade()
            if forChoice==True: r = r-1
            
            for i in range(r):
                if context.indentWithPipes: prefix+='|'
                prefix+="    "
            
        return prefix
    
    def replaceWithVars(msg:str, context: Context):
        
        try:
            i1 = msg.index("%$")
            i2 = msg.index("$%")+2
            varName = msg[i1+2:i2-2]
            varValue = context.vars[varName]
            msg = msg[:i1] + str(varValue) + msg[i2:]
            return Printer.replaceWithVars(msg, context)
        except ValueError:
            pass

        try:
            i1 = msg.index("%=")
            i2 = msg.index("=%")+2
            varName = msg[i1+2:i2-2]
            varValue = context.__dict__[varName]
            msg = msg[:i1] + str(varValue) + msg[i2:]
            return Printer.replaceWithVars(msg, context)
        except ValueError:
            pass
        except KeyError: 
            Printer.printError(varName+ ' is not a default menu variable!', context)
        
        return msg
    
    def print(message:str, context:Context, forChoice = False):
        """Prints a simple message."""
        
        message = Printer.replaceWithVars(message, context)
        
        print (Printer.prepareIndent(context, forChoice) + str(message))
        
        
    def printError(message:str, context:Context):
        """Prints an error message in red, followed by the instructions in the file that led to it."""
        
        message = Printer.replaceWithVars(message, context)
        
        fullMsg = ""
        fullMsg += Fore.RED + Style.BRIGHT + message+"\n"
        fullMsg += "In file: "+context.file
        if context.showTrace:
            fullMsg+= "\n"+str(context.path.instructionsAsStackTrace())
        fullMsg += Style.RESET_ALL + Fore.RESET
        
        print (fullMsg)
        
        context.exception = fullMsg
        
    def printChoice(choiceInstr, context:Context):
    #     print(context.path)
        id = len(context.availableChoices)
        msg = context.choicePrompt
        msg = msg.replace('%$', str(choiceInstr.choiceName))
        msg = msg.replace('%#', str(id))
        
        Printer.print(msg, context, True)
    
    def printExitChoice(context: Context):
        id = len(context.availableChoices)+1
        name = 'Exit'
        msg = context.choicePrompt
        msg = msg.replace('%$', str(name))
        msg = msg.replace('%#', str(id))
        Printer.print(msg, context, False)
        
    def askForInput(context: Context, prompt:str = None):
        if prompt is None: 
            prompt = context.inputPrompt
        prompt = Printer.replaceWithVars(prompt, context)
        
        return input(Printer.prepareIndent(context)+prompt)


from cmdMenuFramework.Instruction import Instruction
from cmdMenuFramework.InstType import InstType
from cmdMenuFramework.Printer import Printer

class Prompt(Instruction):
    
    def __init__(self, context, name, content):
        super(Prompt, self).__init__(context, InstType.prompt, name, content)
        
    def execute(self):
        """Runs the instruction. 
        
        If it contains a dictionnary, executes the first instruction from this dictionnary, and prints its return value.
        If it contains anything else, prints str(value)."""
        
        self.context.path.addInstruction(self)
        
        if type(self.content) == dict:
            if (len(self.content) != 1):
                Printer.printError ("Multiple embedded instructions: use a block instruction instead", self.context)
                self.context.path.goBack()
                return None
            for id in self.content:
                i = Instruction.letterToInstruction(id[:1], id, self.content[id], self.context)
                if i is None: 
                    self.context.path.goBack()
                    return None
                msg = i.execute()
            
        else:   
            msg = self.content
            
        Printer.print(str(msg), self.context)
        
        self.context.path.goBack()
        return msg
    
    
    
    




from enum import Enum

class InstType(Enum):
    prompt = 0
    choice = 1
    question = 2
    action = 3
    set = 4
    get = 5
    setDefault = 6
    getDefault = 7
    input = 8
    output = 9
    name = 10
    block = 11
    loop = 12
    exit = 13
    finish = 14
    wait = 15
    exception = 16
    host = 17
    v = 18
    



from cmdMenuFramework.Path import Path

class Context:
    
    def __init__(self, file = 'yaml/index.yaml'):
        # path is the list of instructions leading to the one executing currently.
        self.path = Path()
        
        # Default menu vars
        self.file = file
        self.controllerFile = 'controllers/ControllerIndex.py'
        self.controllerClass = 'ControllerIndex'
        self.input = None
        self.output = None
        self.userChoice = None
        self.exit = False
        self.exception = None
        self.finish = False
        self.choicePrompt = '%#. %$' 
        self.inputPrompt = 'Your input > '
        self.indent = False
        self.availableChoices = []
        self.showTrace = True
        self.indentWithPipes = False
        
        # User menu vars: 
        self.vars = dict()
        
        
    
    
    
    


from cmdMenuFramework.Instruction import Instruction
from cmdMenuFramework.InstType import InstType

class Value(Instruction):
    
    def __init__(self, context, name, content):
        super(Value, self).__init__(context, InstType.v, name, content)
        
    def execute(self):
        return self.content
    
    
    
    


from cmdMenuFramework.Instruction import Instruction
from cmdMenuFramework.InstType import InstType
from cmdMenuFramework.Printer import Printer

class Name(Instruction):
    
    def __init__(self,  context, name, content):
        super(Name, self).__init__(context, InstType.name, name, content)
        
    def execute(self):
        
        self.context.path.addInstruction(self)
        
        if type(self.content) == dict:
            if (len(self.content) != 1):
                Printer.printError ("Multiple embedded instructions: use a block instruction instead", self.context)
                self.context.path.goBack()
                return None
            for id in self.content:
                i = Instruction.letterToInstruction(id[:1], id, self.content[id], self.context)
                if i is None: 
                    self.context.path.goBack()
                    return None
                name = i.execute()
            
        else:   
            name = self.content
            
        self.context.path.goBack()
        return str(name)


from cmdMenuFramework.Instruction import Instruction
from cmdMenuFramework.InstType import InstType
from cmdMenuFramework.Printer import Printer         
import time
class Wait(Instruction):
    
    def __init__(self, context, name, content):
        super(Wait, self).__init__(context, InstType.wait, name, content)
        
    def execute(self):
        """Runs the instruction. 
        
        If it contains a dictionnary, executes the first instruction from this dictionnary, and prints its return value.
        If it contains anything else, prints str(value)."""
        
        self.context.path.addInstruction(self)
        
        c = None
        
        if self.content is not None:
            if type(self.content) == dict:
                if (len(self.content) != 1):
                    Printer.printError ("Multiple embedded instructions: use a block instruction instead", self.context)
                    self.context.path.goBack()
                    return None
                for id in self.content:
                    i = Instruction.letterToInstruction(id[:1], id, self.content[id], self.context)
                    if i is None: 
                        self.context.path.goBack()
                        return None
                    c = i.execute()
                    if type(c)!=str and type(c)!=int:
                        Printer.printError ("Wait instruction expected either a string or an integer, got "+str(type(c))+" from embedded instruction instead.", self.context)
                        self.context.path.goBack()
                        return None
                    
            elif type(self.content)==str or type(self.content)==int:
                c = self.content
            else: 
                Printer.printError ("Wait instruction expected either a string or an integer, got "+type(self.content)+" instead.", self.context)
                self.context.path.goBack()
                return None
                
                
        if type(c)==str:
            while True:
                if c == "":
                    msg = "Press enter to continue"
                else: 
                    msg = 'Write "'+c+'" to continue'
                written = Printer.askForInput(self.context, msg+" > ")
                if written == c:
                    break;
        
        elif type(c)==int:
            time.sleep(c)
    
        
        self.context.path.goBack()
    
    
    
    


from cmdMenuFramework.Instruction import Instruction
from cmdMenuFramework.InstType import InstType
from cmdMenuFramework.Printer import Printer

class Input(Instruction):
    
    def __init__(self, context, name, content):
        super(Input, self).__init__(context, InstType.input, name, content)
         
    def execute(self):
        self.context.path.addInstruction(self)
        
        customPrompt = None
        
        if self.content is not None: 
            if type(self.content) == str:
                customPrompt = self.content
            
            elif type(self.content) == dict:
                if (len(self.content) != 1):
                    Printer.printError ("Multiple embedded instructions: use a block instruction instead", self.context)
                    self.context.path.goBack()
                    return None
            
                for id in self.content:
                    i = Instruction.letterToInstruction(id[:1], id, self.content[id], self.context)
                    if i is None: 
                        self.context.path.goBack()
                        return None
                    customPrompt = i.execute()
                    if type(customPrompt) != str:  
                        Printer.printError ("Input instruction expected a string, got "+ type(customPrompt) +" from embedded instruction instead", self.context)
                        self.context.path.goBack() 
                        return None
            
            else: 
                Printer.printError('Input instruction takes null or a string as parameter, was given '+type(self.content)+' instead.')
                self.context.path.goBack()
                return None
            
                
        
        output = Printer.askForInput(self.context, customPrompt)
        self.context.input = output  
            
        self.context.path.goBack()
        return output



from abc import ABC, abstractmethod
from cmdMenuFramework.Context import Context

class Instruction:
    
    
    
    def __init__(self, context:Context, type, name, content):
        self.type = type
        self.name = name # the key name for the instruction
        self.content = content # the contents of the instruction
        self.context = context
        
    @abstractmethod
    def execute(self): pass
    
    
    def __str__(self) :
        return str(self.type)+" [ "+self.name + ", " + str(self.content) + " ]"
    
    
    
    
    def letterToInstruction(letter, key, content, context):
        # keep those imports inside, otherwise, there is a circular import error
        from cmdMenuFramework.Prompt        import Prompt
        from cmdMenuFramework.Choice        import Choice
        from cmdMenuFramework.Name          import Name
        from cmdMenuFramework.Question      import Question
        from cmdMenuFramework.Action        import Action
        from cmdMenuFramework.Get           import Get
        from cmdMenuFramework.Set           import Set
        from cmdMenuFramework.SetDefault    import SetDefault
        from cmdMenuFramework.GetDefault    import GetDefault
        from cmdMenuFramework.Input         import Input
        from cmdMenuFramework.Host          import Host
        from cmdMenuFramework.Output        import Output
        from cmdMenuFramework.Exception     import Exception as X
        from cmdMenuFramework.Wait          import Wait
        from cmdMenuFramework.Value         import Value
        from cmdMenuFramework.Exit          import Exit
        from cmdMenuFramework.Finish        import Finish
        from cmdMenuFramework.Block         import Block
        from cmdMenuFramework.Loop          import Loop
        
        switcher = {
            'p': Prompt,
            'c': Choice,
            'n': Name, 
            'q': Question,
            'a': Action,
            's': Set,
            'g': Get,
            '-': SetDefault,
            '<': GetDefault,
            'i': Input, 
            'h': Host,
            'o': Output,
            'x': X,
            'w': Wait,
            'v': Value,
            'e': Exit, 
            'f': Finish,
            'b': Block,
            'l': Loop
        }
        # Get the constructor from switcher dictionary
        constructor = switcher.get(letter)
        
        if constructor is None:
            from Printer import Printer
            Printer.printError("Invalid instruction: "+letter, context)
            return None
        # Construct and return object
        return constructor(context, key, content)
    
    



from cmdMenuFramework.InstType import InstType

class Path:
    
    def __init__(self):
        self.instructions = []
        
    
    def isAtRoot(self):
        return self.instructions == []
    
    def getInstruction( self, index ):
        return self.instructions[index]
            
    def getLastInstruction( self ):
        return self.instructions[-1]
    
    def goBack( self ):
        self.instructions.pop()
        
    def addInstruction( self, instruction ):
        self.instructions.append(instruction)
    
    def getChoicesMade( self ):
        choices = []
        for i in self.instructions:
            if i.type == InstType.choice:
                choices.append(i)
        # # if the last is a choice, it has not yet been choosen, its just declaring itself
        # if choices != []:
        #     if choices[-1].type == InstType.choice:
        #         choices = choices[:-1]
        return choices
    
    def getNbChoicesMade( self ):
        return len(self.getChoicesMade())
    
    
    def instructionsAsStackTrace(self):
        st = ""
        for i in self.instructions:
            st += 'at: '+i.name + "\n"
            if i.type == InstType.host:
                if hasattr(i, 'fileHosted'):
                    st+="In file: "+i.fileHosted+"\n"
        return st
    
    def __str__(self):
        list = 'Path object: [ '
        for item in self.instructions:
            list += str(item)+", \n"
        return  list + " ]"


from cmdMenuFramework.Instruction import Instruction
from cmdMenuFramework.InstType import InstType
from cmdMenuFramework.Printer import Printer
from cmdMenuFramework.Menu import Menu
class Host(Instruction):
    
    def __init__(self, context, name, content):
        super(Host, self).__init__(context, InstType.host, name, content)
         
         
    def execute(self):
        self.context.path.addInstruction(self)
        
        fileName = None
        
        if self.content is not None: 
            if type(self.content) == str:
                fileName = self.content
            
            elif type(self.content) == dict:
                if (len(self.content) != 1):
                    Printer.printError ("Multiple embedded instructions: use a block instruction instead", self.context)
                    self.context.path.goBack()
                    return None
            
                for id in self.content:
                    i = Instruction.letterToInstruction(id[:1], id, self.content[id], self.context)
                    if i is None: 
                        self.context.path.goBack()
                        return None
                    fileName = i.execute()
                    if type(fileName) != str:  
                        Printer.printError ("Host instruction expected a string (yaml file), got "+ type(fileName) +" from embedded instruction instead", self.context)
                        self.context.path.goBack() 
                        return None
            
            else: 
                Printer.printError('Host instruction takes a string as parameter, was given '+type(self.content)+' instead.')
                self.context.path.goBack()
                return None
        else: 
            Printer.printError ("Host instruction recieved null as parameter.", self.context)
            self.context.path.goBack() 
            return None
            
        self.fileHosted = fileName
        embeddedMenu = Menu(fileName, self.context)
        output = embeddedMenu.start()
        
          
            
        self.context.path.goBack()
        return output


from cmdMenuFramework.Instruction import Instruction
from cmdMenuFramework.InstType import InstType

class Block(Instruction):
    
    def __init__(self, context, name, content):
        super(Block, self).__init__(context, InstType.block, name, content)
        
    def execute(self):
        
        self.context.path.addInstruction(self)

        for id in self.content:
            
            if self.context.exit == True:
                self.context.exit = False
                break
            
            i = Instruction.letterToInstruction(id[:1], id, self.content[id], self.context)
            
            if i is None: 
                continue
            i.execute()
        
        self.context.path.goBack()
        return self.context.output


from cmdMenuFramework.Instruction import Instruction
from cmdMenuFramework.InstType import InstType
from cmdMenuFramework.Printer import Printer
class GetDefault(Instruction):
    
    def __init__(self, context, name, content):
        super(GetDefault, self).__init__(context, InstType.getDefault, name, content)
        
    def execute(self):
        self.context.path.addInstruction(self)
        
        # find var name
        if self.content is None:
            self.context.path.goBack()
            return None
        
        if (type(self.content)) == dict:
            if (len(self.content) != 1):
                Printer.printError ("Multiple embedded instructions: use a block instruction instead", self.context)
                self.context.path.goBack()
                return None
            
            for id in self.content:
                i = Instruction.letterToInstruction(id[:1], id, self.content[id], self.context)
                if i is None: 
                    self.context.path.goBack()
                    return None
                varName = i.execute()
                if type(varName) != str:  
                    Printer.printError ("GetDefault instruction expected a variable name (str), got "+ type(varName) +" from embedded instruction instead", self.context)
                    self.context.path.goBack() 
                    return None
                
        elif type(self.content) == str:   
            varName = self.content
        
        
        else: 
            Printer.printError ("GetDefault instruction expected a variable name (str), got "+ type(self.content) +" instead", self.context)
            self.context.path.goBack() 
            return None
        
        
        # get var from context
        if varName is None:
            self.context.path.goBack() 
            return None
        
        allDefDic = {
                'file' : self.context.file,
                'controllerFile' : self.context.controllerFile,
                'controllerClass' : self.context.controllerClass,
                'input' : self.context.input,
                'output': self.context.output,
                'exit' : self.context.exit,
                'exception' : self.context.exception,
                'finish' : self.context.finish,
                'choicePrompt' : self.context.choicePrompt,
                'inputPrompt' : self.context.inputPrompt,
                'indent' : self.context.indent,
                'userChoice': self.context.userChoice,
                'availableChoices' : self.context.availableChoices,
                'path': self.context.path,
                'showTrace': self.context.showTrace,
                'indentWithPipes': self.context.indentWithPipes
            }
        
        if varName == '*':
            self.context.path.goBack()
            return allDefDic
        
        try :
            var = allDefDic[varName]
        except KeyError :
            Printer.printError ('"'+varName+"\" is not a default menu variable! Oh no!  :'(", self.context)
            self.context.path.goBack()
            return None
            
        self.context.path.goBack()
        return var
    
   


from distutils.core import setup
setup(
  name = 'cmdMenuFramework',         
  packages = ['cmdMenuFramework'],  
  version = '1.0.3',
  license='MIT',
  description = 'This is a library/framework specialised in creating a command line menus, through the use of yaml files.',   # Give a short description about your library
  author = 'ARAGON Esteban',             
  author_email = 'aragonstban@gmail.com', 
  url = 'https://github.com/Este2013/command_line_menu_python',   
  download_url = 'https://github.com/Este2013/command_line_menu_python/archive/refs/tags/v1.0.3.tar.gz', 
  keywords = ['command', 'cmd', 'line', 'menu', 'framework'], 
  install_requires=[  
          'colorama',
          'pyyaml',
          'importlib'
      ],
  classifiers=[
    'Development Status :: 4 - Beta',   # Choices are ["3 - Alpha", "4 - Beta", "5 - Production/Stable"]
    'Intended Audience :: Developers',     
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License', 
    'Programming Language :: Python :: 3', 
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9'
  ],
)




from cmdMenuFramework.Printer import Printer
from cmdMenuFramework.Context import Context
from cmdMenuFramework.Instruction import Instruction
import yaml

class Menu:
    
    
    def __init__(self, file = 'yaml/index.yaml', context:Context = None):
        self.context = Context(file)
        if context is not None:
            self.context.vars = context.vars
            self.context.controllerFile = context.controllerFile
            self.context.controllerClass = context.controllerClass
            self.context.input = context.input
            self.context.exit = context.exit
            self.context.exception = context.exception
            self.context.finish = context.finish
            self.context.choicePrompt = context.choicePrompt
            self.context.inputPrompt = context.inputPrompt
            self.context.indent = context.indent
            self.context.showTrace = context.showTrace
            self.context.indentWithPipes = context.indentWithPipes
            self.context.availableChoices = context.availableChoices
            self.context.path = context.path
            
        
    
    def start(self):
        # loads then executes the base instructions, one by one.
        try: 
            with open(self.context.file) as f:
                self.instructionsAsDict = yaml.load(f, Loader=yaml.FullLoader)
                self.execute()
                return self.context.output
        except FileNotFoundError: 
            Printer.printError("File of menu to launch does not exist: "+str(self.context.file), self.context)
    
    
    def execute(self):
        for id in self.instructionsAsDict:
            
            if self.context.exit == True:
                self.context.exit = False
                break
                
            
            i = Instruction.letterToInstruction(id[:1], id, self.instructionsAsDict[id], self.context)
            
            if i is None: 
                continue
            i.execute()
    
    
    
    
    
    


from cmdMenuFramework.Instruction import Instruction
from cmdMenuFramework.InstType import InstType
from cmdMenuFramework.Printer import Printer
import importlib
import importlib.util

class Action(Instruction):
    
    def __init__(self, context, name, content):
        super(Action, self).__init__(context, InstType.action, name, content)
           
    def execute(self):
        self.context.path.addInstruction(self)
        
        if (typ := type(self.content)) == dict:
            if (len(self.content) != 1):
                Printer.printError ("Multiple embedded instructions: use a block instruction instead", self.context)
                self.context.path.goBack()
                return None
            
            for id in self.content:
                i = Instruction.letterToInstruction(id[:1], id, self.content[id], self.context)
                if i is None: 
                    self.context.path.goBack()
                    return None
                actionName = i.execute()
                if type(actionName) != str:  
                    Printer.printError ("Action instruction expected a method name (str), got "+ type(actionName) +" from embedded instruction instead", self.context)
                    self.context.path.goBack() 
                    return None
                
        elif typ == str:   
            actionName = self.content
            
        else: 
            Printer.printError ("Action instruction expected a method name (str), got "+str(typ)+" instead", self.context)
            self.context.path.goBack() 
            return None
        
        
        # execute action from controller
        try:
            
            spec = importlib.util.spec_from_file_location("controllingModule", self.context.controllerFile)
            foo = importlib.util.module_from_spec(spec)
            spec.loader.exec_module(foo)  
            
            if self.context.controllerClass is None:
                klass = foo
            else: 
                klass = foo.__dict__[self.context.controllerClass]
            method =  klass.__dict__[actionName]
            
            output = method(self.context)
            self.context.path.goBack()
            return output
            
        except FileNotFoundError as e:
            Printer.printError('There is no file named: '+self.context.controllerFile, self.context)
            self.context.path.goBack()
            return None    
        except KeyError:
            Printer.printError('There is either: \n    -no class named: '+self.context.controllerClass+' in the file: '+self.context.controllerFile
                +'\n    -or no method named: '+actionName+' in this class.', self.context)
            self.context.path.goBack()
            return None
            
        
        


from cmdMenuFramework.Instruction import Instruction
from cmdMenuFramework.InstType import InstType
from cmdMenuFramework.Printer import Printer

class Output(Instruction):
    
    def __init__(self, context, name, content):
        super(Output, self).__init__(context, InstType.output, name, content)
         
    
    def execute(self):
        self.context.path.addInstruction(self)
        
        output = None
        
        if self.content is not None: 
            if type(self.content) == dict:
                if (len(self.content) != 1):
                    Printer.printError ("Multiple embedded instructions: use a block instruction instead", self.context)
                    self.context.path.goBack()
                    return None
            
                for id in self.content:
                    i = Instruction.letterToInstruction(id[:1], id, self.content[id], self.context)
                    if i is None: 
                        self.context.path.goBack()
                        return None
                    output = i.execute()
            
            else: 
                output = self.content
            
        self.context.output = output
        self.context.path.goBack()
        return output


from cmdMenuFramework.Instruction import Instruction
from cmdMenuFramework.InstType import InstType
from cmdMenuFramework.Printer import Printer

class Question(Instruction):
    
    def __init__(self, context, name, content):
        super(Question, self).__init__(context, InstType.question, name, content)
        
    def execute(self):
        
        self.context.path.addInstruction(self)
        
        # set default parameters
        noExit = False
        ifError = None # contains the instruction
        ifExit = None
        
        if self.content is not None: # set custom parameters
            if type(self.content) != dict:
                Printer.printError("Question instruction expects either null or a dictionnary", self.context)
            
            # CHECK FOR THE DIFFERENT PARAMS IF THERE ARE
            for p in self.content:
                if p == 'noExit':
                    #test if we have a boolean
                    if type(self.content[p]) == bool:
                        noExit = self.content[p]
                        
                    #test if we have an embedded instruction 
                    elif type(self.content[p]) == dict:
                        # test if there is only one 
                        if (len(self.content[p])) != 1 :
                            Printer.printError ("Multiple embedded instructions: use a block instruction instead", self.context)
                            self.context.path.goBack()
                            return None
                        for id in self.content[p]: # create, execute, and check return value of embedded instruction
                            i = Instruction.letterToInstruction(id[:1], id, self.content[id], self.context)
                            if i is None: 
                                self.context.path.goBack()
                                return None
                            boolean = i.execute()
                            if type(boolean) != bool:
                                self.context.path.goBack()
                                Printer.printError("noExit parameter expects a boolean, got "+type(boolean)+" instead, from an embedded instruction", self.context)
                                return None
                            else: 
                                noExit = boolean
                    else: 
                        Printer.printError("noExit parameter expects a boolean, got "+type(self.content[p])+" instead", self.context)
                
                elif p == 'ifError': # ifError accepts only one instruction 
                    if self.content[p] is not None:
                        if type(self.content[p]) == dict:
                            if (len(self.content[p])) != 1 :
                                Printer.printError ("Multiple embedded instructions: use a block instruction instead", self.context)
                                self.context.path.goBack()
                                return None

                            for id in self.content[p]:
                                i = Instruction.letterToInstruction(id[:1], id, self.content[p][id], self.context)
                                if i is None: 
                                    self.context.path.goBack()
                                    return None
                                ifError = i
                        else : 
                            Printer.printError("ifError parameter expects an embedded instruction, got "+type(self.content[p])+" instead", self.context)
                
                elif p == 'ifExit': # onExit accepts only one instruction 
                    if self.content[p] is not None:
                        if type(self.content[p]) == dict:
                            if (len(self.content[p])) != 1 :
                                Printer.printError ("Multiple embedded instructions: use a block instruction instead", self.context)
                                self.context.path.goBack()
                                return None

                            for id in self.content[p]:
                                i = Instruction.letterToInstruction(id[:1], id, self.content[p][id], self.context)
                                if i is None: 
                                    self.context.path.goBack()
                                    return None
                                ifExit = i
                        else : 
                            Printer.printError("onExit parameter expects an embedded instruction, got "+type(self.content[p])+" instead", self.context)
                
                else: 
                    Printer.printError("Unrecognized parameter for a Question instruction: " + str(p), self.context)
                    
        # if the noExit == False: propose choice for exit
        if noExit==False:
            Printer.printExitChoice(self.context)
        
        # now ask for a choice
        choice = Printer.askForInput(self.context)
        self.context.userChoice = choice
        
        # interpret user response
        askForExit = False
        errorEncountered = None
        
        # if the choice is a number between 1 and however many choices there are
        maxId = len(self.context.availableChoices)+1
        if noExit==True: maxId -= 1 
        
        if choice.isnumeric() and (id:=int(choice))>0 and id<= maxId:
            # there is nothing to do if we ask to exit the choice, but in other case: 
            if id < len(self.context.availableChoices)+1:
                choosen = self.context.availableChoices[id-1]
                self.context.availableChoices = [] 
                choosen.executeContent()
                
            elif ifExit is not None:
                ifExit.execute()
                
        else:
            if ifError is not None:
                ifError.execute()
        
        self.context.availableChoices = []   
        self.context.path.goBack()
        return choice
        
    